import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';
import 'package:ngpack/ngpack.dart';
import 'package:test/test.dart';

void main() {
  test('xorEncode', () {
    final data = 'hello world';
    final bytes = Uint8List.fromList(data.codeUnits);
    final actual = XorCodec(Keys[KnownKey.Key5b6d]!).encode(bytes);
    final expected = [44, 187, 16, 237, 151, 51, 9, 79, 216, 227, 44];
    expect(actual, containsAllInOrder(expected));
  });
  test('xorDecode', () {
    final data =
        Uint8List.fromList([44, 187, 16, 237, 151, 51, 9, 79, 216, 227, 44]);
    final actual = utf8.decode(XorCodec(Keys[KnownKey.Key5b6d]!).decode(data));
    expect(actual, equals('hello world'));
  });
  test('mapEncode', () {
    final data = {
      'integer': 5,
      'array': [
        null,
        0,
        true,
        3.14,
        'string',
        {'key': 'value'}
      ]
    };
    final actual = ggmap.encode(data);
    final expected = File('test/mapEncoded').readAsBytesSync();
    expect(actual, containsAllInOrder(expected));
  });
  test('mapDecode', () {
    final data = File('test/mapEncoded').readAsBytesSync();
    final actual = GGMapCodec().decode(data);
    final expected = {
      'integer': 5,
      'array': [
        null,
        0,
        1,
        3.14,
        'string',
        {'key': 'value'}
      ]
    };
    expect(actual, equals(expected));
  });
  test('ggpackEncode', () {
    final encoder = GGPackFileEncoder(Keys[KnownKey.Key56ad]!)
      ..addContent('hello.txt', 'hello world')
      ..close();
    final actual = encoder.toBytes();
    final expected = File('test/ggpackEncoded.ggpack').readAsBytesSync();
    expect(actual, equals(expected));
  });
  test('ggpackDecode', () {
    final decoder = GGPackFileDecoder.fromFile(
        'test/ggpackEncoded.ggpack', Keys[KnownKey.Key56ad]!);
    expect(decoder, containsAllInOrder([GGPackEntry('hello.txt', 8, 11)]));
    final content = utf8.decode(decoder.extract('hello.txt'));
    expect(content, equals('hello world'));
  });
  test('bnutDecode', () {
    final data = [0x63, 0xca, 0x67, 0x6f, 0x23, 0x4e];
    final decoded = bnut.decode(data);
    expect(decoded, equals('secret'));
  });
}
